//directionAO.glsl


float getCosine(vec4 currentPos, vec4 samplePos, vec4 currentPixelNormalVector, float bias, float radius)
{
		vec3 d = normalize(samplePos.xyz - currentPos.xyz);	 
		return  dot( d  , normalize(currentPixelNormalVector.xyz)) - bias;
}



float getEqualDistanceWeight(float dist, float threshold)
{
	return 1;
}

float getBinaryDistanceWeight(float dist, float threshold)
{
	return dist< threshold ? 1: 0;
}

float getWendlandDistanceWeight(float dist, float threshold)
{
	return  wendland(threshold , dist);
}

float getDistanceWeight(float dist, float threshold)
{
	//return getEqualDistanceWeight( dist, threshold);
	return getBinaryDistanceWeight(dist,threshold);
	//return  getWendlandDistanceWeight(dist, threshold);
}

float getSolidAngle(vec4 currentPos, vec4 samplePos, vec4 currentPixelNormalVector, float bias, float radius)
{
		vec3 d = samplePos.xyz - currentPos.xyz;	 
		float l = length(d);
		
		return getCosine(currentPos, samplePos, currentPixelNormalVector, bias, radius)  * getDistanceWeight( l, radius);
}



float computeAO(const vec4 currentPos, const vec4 samplePos,const vec4 currentPixelNormalVector, float bias, float radius) {
		return getSolidAngle(currentPos, samplePos, currentPixelNormalVector, bias, radius);
}


float marchRay(vec4 currentPosition, vec4 currentPixelNormalVector, vec2 direction, int stepCount, float radius, float offset, float bias, sampler2D mvPositionMap, mat4 In_ProjectionMatrix)
{
	vec4 pSample = getSamplePosition(currentPosition, vec2(direction) * radius);
    vec4 projectedP = In_ProjectionMatrix * pSample;
    vec2 screenPos = (projectedP.xy / projectedP.w + vec2 (1) ) * 0.5;
    vec2 screenPosOffset = screenPos - gl_TexCoord[0].st;

	float ao = 0;
	for( int i = 1; i <= stepCount; i++) {
	
		vec2 screenPosT = gl_TexCoord[0].st + screenPosOffset * float(i + offset)/ float(stepCount);
        vec4 samplePos = texture (mvPositionMap,  screenPosT);
        if(samplePos.z < 0){
            ao = max(ao, computeAO(currentPosition, samplePos, currentPixelNormalVector, bias, radius));
          
		}
		
	}

	return ao;
}



float marchDoubleRay(vec4 currentPosition, vec4 currentNormalVector, vec2 direction, int stepCount, float radius, float offset, float bias, sampler2D mvPositionMap, mat4 In_ProjectionMatrix)
{

	float ao0 = marchRay(currentPosition, currentNormalVector, direction, stepCount, radius, offset, bias, mvPositionMap, In_ProjectionMatrix);
	float ao1 = marchRay(currentPosition, currentNormalVector, -direction, stepCount, radius, offset, bias, mvPositionMap, In_ProjectionMatrix);


	//return max(ao1,ao0) * 0.5;
	return (ao1 + ao0) * 0.5;
}








